@@ -1,5 +1,5 @@
 package Starch::Manager;
-$Starch::Manager::VERSION = '0.03';
+$Starch::Manager::VERSION = '0.04';
 =head1 NAME
 
 Starch::Manager - Entry point for accessing Starch state objects.
@@ -24,6 +24,8 @@ L<Starch::Manual/METHOD PROXIES>.
 use Starch::State;
 use Starch::Util qw( croak );
 use Storable qw( freeze dclone );
+use Scalar::Util qw( refaddr );
+use Digest::SHA qw( sha1_hex );
 
 use Types::Standard -types;
 use Types::Common::String -types;
@@ -135,29 +137,20 @@ L<Starch::Manual/PLUGINS>.
 
 =head2 namespace
 
-The root namespace to put starch data in.  In most cases this is just
-prepended to the state ID and used as the key for storing the state
-data.  Defaults to C<starch-state>.
+The root array ref namespace to put starch data in.  In most cases this is
+just prepended to the state ID and used as the key for storing the state
+data.  Defaults to C<['starch-state']>.
+
+If you are using the same store for independent application states you
+may want to namespace them so that you can easly identify which application
+a particular state belongs to when looking in the store.
 
 =cut
 
 has namespace => (
     is      => 'ro',
-    isa     => NonEmptySimpleStr,
-    default => 'starch-state',
-);
-
-=head2 key_separator
-
-Used by L</stringify_key> to combine the state namespace
-and ID.  Defaults to C<:>.
-
-=cut
-
-has key_separator => (
-    is      => 'ro',
-    isa     => NonEmptySimpleStr,
-    default => ':',
+    isa     => ArrayRef[ NonEmptySimpleStr ],
+    default => sub{ ['starch-state'] },
 );
 
 =head2 expires_state_key
@@ -199,22 +192,34 @@ has created_state_key => (
     default => '__STARCH_CREATED__',
 );
 
-=head2 invalid_state_key
+=head2 no_store_state_key
 
-This key is used by stores to mark state data as invalid,
-and when set in the state will disable the state from being
-written to the store.  Defaults to C<__STARCH_INVALID__>.
+This key is used by stores to mark state data as not to be
+stored.  Defaults to C<__STARCH_NO_STORE__>.
 
 This is used by the L<Starch::Plugin::LogStoreExceptions> and
 L<Starch::Plugin::ThrottleStore> plugins to avoid losing state
-datain the store when errors or throttling is encountered.
+data in the store when errors or throttling is encountered.
 
 =cut
 
-has invalid_state_key => (
+has no_store_state_key => (
     is      => 'ro',
     isa     => NonEmptySimpleStr,
-    default => '__STARCH_INVALID__',
+    default => '__STARCH_NO_STORE__',
+);
+
+=head2 dirty_state_key
+
+This key is used to artificially mark as state as dirty by incrementing
+the value of this key.  Used by L<Starch::State/mark_dirty>.
+
+=cut
+
+has dirty_state_key => (
+    is      => 'ro',
+    isa     => NonEmptySimpleStr,
+    default => '__STARCH_DIRTY__',
 );
 
 =head1 ATTRIBUTES
@@ -263,27 +268,28 @@ sub state {
     );
 }
 
-=head2 stringify_key
+=head2 state_id_seed
 
-    my $store_key = $starch->stringify_key(
-        $state_id,
-        \@namespace,
-    );
-
-This method is used by stores that store and lookup data by
-a string (all of them at this time).  It combines the state
-ID with the L</namespace> of the key data for the store
-request.
+Returns a fairly unique string used for seeding L<Starch::State/id>.
 
 =cut
 
-sub stringify_key {
-    my ($self, $id, $namespace) = @_;
-    return join(
-        $self->key_separator(),
-        @$namespace,
-        $id,
-    );
+my $counter = 0;
+sub state_id_seed {
+    my ($self) = @_;
+    return join( '', ++$counter, time, rand, $$, {}, refaddr($self) )
+}
+
+=head2 generate_state_id
+
+Generates and returns a new state ID which is a SHA-1 hex
+digest of calling L</state_id_seed>.
+
+=cut
+
+sub generate_state_id {
+    my ($self) = @_;
+    return sha1_hex( $self->state_id_seed() );
 }
 
 =head2 clone_data
