@@ -1,5 +1,5 @@
 package Starch::Plugin::LogStoreExceptions;
-$Starch::Plugin::LogStoreExceptions::VERSION = '0.03';
+$Starch::Plugin::LogStoreExceptions::VERSION = '0.04';
 =head1 NAME
 
 Starch::Plugin::LogStoreExceptions - Turn Starch store exceptions into log messages.
@@ -50,7 +50,7 @@ foreach my $method (qw( set get remove )) {
                 ref($self), $method, $_,
             );
             return {
-                $self->manager->invalid_state_key() => 1,
+                $self->manager->no_store_state_key() => 1,
             } if $method eq 'get';
             return;
         };
