@@ -1,5 +1,5 @@
 package Starch::State;
-$Starch::State::VERSION = '0.03';
+$Starch::State::VERSION = '0.04';
 =head1 NAME
 
 Starch::State - The Starch state object.
@@ -18,10 +18,8 @@ This is the state class used by L<Starch::Manager/state>.
 
 =cut
 
-use Scalar::Util qw( refaddr );
 use Types::Standard -types;
 use Types::Common::String -types;
-use Digest::SHA qw( sha1_hex );
 use Starch::Util qw( croak );
 
 use Moo;
@@ -72,7 +70,7 @@ has id => (
 sub _build_id {
     my ($self) = @_;
     return $self->_existing_id() if $self->_has_existing_id();
-    return $self->generate_id();
+    return $self->manager->generate_state_id();
 }
 
 =head1 ATTRIBUTES
@@ -97,10 +95,12 @@ sub _build_original_data {
     return {} if !$self->in_store();
 
     my $manager = $self->manager();
-    my $data = $manager->store->get( $self->id(), [$manager->namespace()] );
-    $data = {} if !$data;
+    my $data = $manager->store->get( $self->id(), $manager->namespace() );
 
-    return $data;
+    return $data if $data;
+
+    $self->_set_in_store( 0 );
+    return {};
 }
 
 =head2 data
@@ -197,6 +197,10 @@ sub _build_created {
 Returns true if the state is expected to exist in the store
 (AKA, if the L</id> argument was specified or L</save> was called).
 
+Note that the value of this attribute may change after L</data>
+is called which will set this to false if the store did not have
+the data for the state.
+
 =cut
 
 has in_store => (
@@ -254,7 +258,7 @@ are called.
 
 =head2 is_saved
 
-Returns true if the state was saved and is not dirty.
+Returns true if the state was saved, is L</in_store>, and is not L</is_dirty>.
 
 =cut
 
@@ -267,6 +271,7 @@ has _save_was_called => (
 sub is_saved {
     my ($self) = @_;
     return 0 if !$self->_save_was_called();
+    return 0 if !$self->in_store();
     return 0 if $self->is_dirty();
     return 1;
 }
@@ -307,7 +312,7 @@ sub force_save {
 
     $manager->store->set(
         $self->id(),
-        [$manager->namespace()],
+        $manager->namespace(),
         $data,
         $self->expires(),
     );
@@ -350,8 +355,6 @@ Just like L</reload>, but reloads even if the state L</is_dirty>.
 sub force_reload {
     my ($self) = @_;
 
-    return if $self->is_deleted();
-
     $self->_clear_original_data();
     $self->_clear_data();
 
@@ -361,15 +364,15 @@ sub force_reload {
 =head2 mark_clean
 
 Marks the state as not L</is_dirty> by setting L</original_data> to
-L</data>.
+L</data>.  This is a potentially destructive method to call as data
+may be lost since L</save> will silently not save the data if it
+appears clean.
 
 =cut
 
 sub mark_clean {
     my ($self) = @_;
 
-    return if $self->is_deleted();
-
     $self->_set_original_data(
         $self->manager->clone_data( $self->data() ),
     );
@@ -377,6 +380,25 @@ sub mark_clean {
     return;
 }
 
+=head2 mark_dirty
+
+Increments the L<Starch::Manager/dirty_state_key> value in
+L</data>, which causes the state to be considered dirty.
+
+=cut
+
+sub mark_dirty {
+    my ($self) = @_;
+
+    my $key = $self->manager->dirty_state_key();
+
+    my $counter = $self->data->{ $key };
+    $counter = ($counter || 0) + 1;
+    $self->data->{ $key } = $counter;
+
+    return;
+}
+
 =head2 rollback
 
 Sets L</data> to L</original_data>.
@@ -386,8 +408,6 @@ Sets L</data> to L</original_data>.
 sub rollback {
     my ($self) = @_;
 
-    return if $self->is_deleted();
-
     $self->_set_data(
         $self->manager->clone_data( $self->original_data() ),
     );
@@ -422,7 +442,7 @@ sub force_delete {
     my ($self) = @_;
 
     my $manager = $self->manager();
-    $manager->store->remove( $self->id(), [$manager->namespace()] );
+    $manager->store->remove( $self->id(), $manager->namespace() );
 
     $self->_set_original_data( {} );
     $self->_set_data( {} );
@@ -462,28 +482,19 @@ sub set_expires {
     return;
 }
 
-=head2 hash_seed
+=head2 reset_expires
 
-Returns a fairly unique string used for seeding L</id>.
+Sets this state's expires to L<Starch::Manager/expires>, overriding
+and custom expires set on this state.
 
 =cut
 
-my $counter = 0;
-sub hash_seed {
+sub reset_expires {
     my ($self) = @_;
-    return join( '', ++$counter, time, rand, $$, {}, refaddr($self) )
-}
 
-=head2 generate_id
+    $self->set_expires( $self->manager->expires() );
 
-Generates and returns a new state ID which is a SHA-1 hex
-digest of calling L</hash_seed>.
-
-=cut
-
-sub generate_id {
-    my ($self) = @_;
-    return sha1_hex( $self->hash_seed() );
+    return;
 }
 
 =head2 reset_id
@@ -501,7 +512,7 @@ sub reset_id {
     # Remove the data for the current state ID.
     if ($self->in_store()) {
         my $manager = $self->manager();
-        $manager->store->remove( $self->id(), [$manager->namespace()] );
+        $manager->store->remove( $self->id(), $manager->namespace() );
     }
 
     # Ensure that future calls to id generate a new one.
