@@ -1,5 +1,5 @@
 package Test::Starch;
-$Test::Starch::VERSION = '0.03';
+$Test::Starch::VERSION = '0.04';
 =head1 NAME
 
 Test::Starch - Test core features of starch.
@@ -118,6 +118,18 @@ sub test_manager {
     my $starch = $self->new_manager();
 
     subtest 'core tests for ' . ref($starch) => sub{
+        subtest state_id_seed => sub{
+            isnt( $starch->state_id_seed(), $starch->state_id_seed(), 'two seeds are not the same' );
+        };
+
+        subtest generate_state_id => sub{
+            isnt(
+                $starch->generate_state_id(),
+                $starch->generate_state_id(),
+                'two generated ids are not the same',
+            );
+        };
+
         subtest clone_data => sub{
             my $old_data = { foo=>32, bar=>[1,2,3] };
             my $new_data = $starch->clone_data( $old_data );
@@ -126,8 +138,15 @@ sub test_manager {
 
             isnt( "$old_data->{bar}", "$new_data->{bar}", 'clone data structure has different reference' );
         };
-    };
 
+        subtest is_data_diff => sub{
+            my $old_data = { foo=>1 };
+            my $new_data = { foo=>2 };
+
+            ok( $starch->is_data_diff($old_data, $new_data), 'is diff' );
+            ok( (!$starch->is_data_diff($old_data, $old_data)), 'is not diff' );
+        };
+    };
 
     return;
 }
@@ -147,11 +166,12 @@ sub test_state {
         subtest id => sub{
             my $state1 = $starch->state();
             my $state2 = $starch->state();
-            my $state3 = $starch->state( '1234' );
+            my $id = $starch->generate_state_id();
+            my $state3 = $starch->state( $id );
 
-            like( $state1->id(), qr{^\S+$}, 'ID looks good' );
+            ok( $starch->state_id_type->check($id), 'ID looks good' );
             isnt( $state1->id(), $state2->id(), 'two generated state IDs are not the same' );
-            is( $state3->id(), '1234', 'custom ID was used' );
+            is( $state3->id(), $id, 'custom ID was used' );
         };
 
         subtest expires => sub{
@@ -184,8 +204,14 @@ sub test_state {
             my $state1 = $starch->state();
             my $state2 = $starch->state( $state1->id() );
 
-            is( $state1->in_store(), 0, 'new state is_new' );
-            is( $state2->in_store(), 1, 'existing state is not is_new' );
+            is( $state1->in_store(), 0, 'new state is not in_store' );
+            is( $state2->in_store(), 1, 'existing state is in_store' );
+
+            my $id = $starch->generate_state_id();
+            my $state3 = $starch->state( $id );
+            is( $state3->in_store(), 1, 'existing state is in_store' );
+            $state3->data();
+            is( $state3->in_store(), 0, 'state is no longer in_store when data was not found' );
         };
 
         subtest is_deleted => sub{
@@ -284,6 +310,13 @@ sub test_state {
             is( $state->data->{foo}, 6934, 'data is intact' );
         };
 
+        subtest mark_dirty => sub{
+            my $state = $starch->state();
+            is( $state->is_dirty(), 0, 'is not dirty' );
+            $state->mark_dirty();
+            is( $state->is_dirty(), 1, 'is dirty' );
+        };
+
         subtest rollback => sub{
             my $state = $starch->state();
             $state->data->{foo} = 6934;
@@ -328,19 +361,18 @@ sub test_state {
             is( $state->expires(), 111, 'custom expires was saved' );
         };
 
-        subtest hash_seed => sub{
+        subtest reset_expires => sub{
+            my $starch = $self->new_manager( expires=>111 );
             my $state = $starch->state();
-            isnt( $state->hash_seed(), $state->hash_seed(), 'two hash seeds are not the same' );
-        };
-
-        subtest generate_id => sub{
-            my $state = $starch->state();
-
-            isnt(
-                $state->generate_id(),
-                $state->generate_id(),
-                'two generated ids are not the same',
-            );
+            is( $state->expires(), 111, 'state got default expires' );
+            $state->set_expires( 666 );
+            $state->force_save();
+            $state = $starch->state( $state->id() );
+            is( $state->expires(), 666, 'expires persisted' );
+            $state->reset_expires();
+            $state->force_save();
+            $state = $starch->state( $state->id() );
+            is( $state->expires(), 111, 'state expires was reset' );
         };
 
         subtest reset_id => sub{
@@ -407,6 +439,47 @@ sub test_store {
             );
             is( $starch->store->max_expires(), 67, 'store max_expires explicitly set' );
         };
+
+        subtest class_name => sub{
+            # Add a random plugin so the real class name has the __WITH__ bit.
+            my $starch = $self->new_manager( store=>{class=>'::Memory'}, plugins=>['::LogStoreExceptions'] );
+            is( $starch->store->base_class_name(), 'Starch::Store::Memory', 'base_class_name' );
+            is( $starch->store->short_class_name(), 'Store::Memory', 'short_class_name' );
+            is( $starch->store->short_store_class_name(), 'Memory', 'short_store_class_name' );
+        };
+
+        subtest new_sub_store => sub{
+            my $sub_store1 = $store->new_sub_store( class=>'::Memory', max_expires=>12 );
+            isa_ok( $sub_store1, 'Starch::Store::Memory' );
+            is( $sub_store1->manager(), $store->manager(), 'sub store has same manager as parent store' );
+            my $sub_store2 = $sub_store1->new_sub_store( class=>'::Memory' );
+            is( $sub_store2->max_expires(), 12, 'sub store has max_expires from parent store' );
+        };
+
+        subtest calculate_expires => sub{
+            my $store = $store->new_sub_store( class=>'::Memory', max_expires => 10 );
+            is( $store->calculate_expires( 5 ), 5, 'expires less than max_expires' );
+            is( $store->calculate_expires( 15 ), 10, 'expires more than max_expires' );
+        };
+
+        subtest stringify_key => sub{
+            is( $store->stringify_key( '1234', ['foo'] ), 'foo:1234', 'basic' );
+            is( $store->stringify_key( '1234', ['foo', 'bar'] ), 'foo:bar:1234', 'deep' );
+            is( $store->stringify_key( '1234', [] ), '1234', 'empty' );
+
+            my $store = $store->new_sub_store( class=>'::Memory', key_separator=>'-' );
+            is( $store->stringify_key( '1234', ['foo', 'bar'] ), 'foo-bar-1234', 'custom key_separator' );
+        };
+
+        subtest reap_expired => sub{
+            my $store = $store->new_sub_store( class=>'::Memory' );
+            ok( (!$store->can_reap_expired()), 'expiration reaping is disabled' );
+            like(
+                exception { $store->reap_expired() },
+                qr{does not support expired state reaping},
+                'reap_expired failed',
+            );
+        };
     };
 
     return;
